@@ -253,6 +253,12 @@ $example->Level('20%x');
 $example->Clamp();
 push(@$images,$example);
 
+print "Linear stretch...\n";
+$example=$model->Clone();
+$example->Label('Linear Stretch');
+$example->LinearStretch('5x5');
+push(@$images,$example);
+
 print "Median Filter...\n";
 $example=$model->Clone();
 $example->Label('Median Filter');
@@ -334,10 +340,10 @@ $example->Label('Quantize');
 $example->Quantize();
 push(@$images,$example);
 
-print "Radial Blur...\n";
+print "Rotational Blur...\n";
 $example=$model->Clone();
-$example->Label('Radial Blur');
-$example->RadialBlur(10);
+$example->Label('Rotational Blur');
+$example->RotationalBlur(10);
 push(@$images,$example);
 
 print "Raise...\n";
