@@ -18,7 +18,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -111,7 +111,7 @@
 */
 typedef struct _BMPInfo
 {
-  unsigned int
+  unsigned long
     file_size,
     ba_offset,
     offset_bits,
@@ -125,7 +125,7 @@ typedef struct _BMPInfo
     planes,
     bits_per_pixel;
 
-  unsigned int
+  unsigned long
     compression,
     image_size,
     x_pixels,
@@ -137,7 +137,7 @@ typedef struct _BMPInfo
     alpha_mask,
     colors_important;
 
-  int
+  long
     colorspace;
 
   PrimaryInfo
@@ -218,7 +218,10 @@ static MagickBooleanType DecodeImage(Image *image,const size_t compression,
   q=pixels+(size_t) image->columns*image->rows;
   for (y=0; y < (ssize_t) image->rows; )
   {
-    if ((p < pixels) || (p >= q))
+    MagickBooleanType
+      status;
+
+    if ((p < pixels) || (p > q))
       break;
     count=ReadBlobByte(image);
     if (count == EOF)
@@ -238,8 +241,8 @@ static MagickBooleanType DecodeImage(Image *image,const size_t compression,
         else
           {
             for (i=0; i < (ssize_t) count; i++)
-              *p++=(unsigned char)
-                ((i & 0x01) != 0 ? (byte & 0x0f) : ((byte >> 4) & 0x0f));
+              *p++=(unsigned char) ((i & 0x01) != 0 ? (byte & 0x0f) :
+                ((byte >> 4) & 0x0f));
           }
         x+=count;
       }
@@ -249,6 +252,8 @@ static MagickBooleanType DecodeImage(Image *image,const size_t compression,
           Escape mode.
         */
         count=ReadBlobByte(image);
+        if (count == EOF)
+          break;
         if (count == 0x01)
           return(MagickTrue);
         switch (count)
@@ -306,12 +311,14 @@ static MagickBooleanType DecodeImage(Image *image,const size_t compression,
           }
         }
       }
-    if (SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,image->rows) == MagickFalse)
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
+    if (status == MagickFalse)
       break;
   }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
-  return(MagickTrue);
+  return(y < (ssize_t) image->rows ? MagickFalse : MagickTrue);
 }
 
 /*
@@ -590,12 +597,12 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
     if (image->debug != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-         "  File_size in header:  %u bytes",bmp_info.file_size);
+         "  File_size in header:  %lu bytes",bmp_info.file_size);
 
     bmp_info.offset_bits=ReadBlobLSBLong(image);
     bmp_info.size=ReadBlobLSBLong(image);
     if (image->debug != MagickFalse)
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  BMP size: %u",
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  BMP size: %lu",
         bmp_info.size);
     if (bmp_info.size == 12)
       {
@@ -691,20 +698,19 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
               default:
               {
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                  "  Compression: UNKNOWN (%u)",bmp_info.compression);
+                  "  Compression: UNKNOWN (%lu)",bmp_info.compression);
               }
             }
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "  Number of colors: %u",bmp_info.number_colors);
+              "  Number of colors: %lu",bmp_info.number_colors);
           }
         bmp_info.red_mask=ReadBlobLSBLong(image);
         bmp_info.green_mask=ReadBlobLSBLong(image);
         bmp_info.blue_mask=ReadBlobLSBLong(image);
         if (bmp_info.size > 40)
           {
-
             double
-              sum;
+              gamma;
 
             /*
               Read color management information.
@@ -725,24 +731,27 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             bmp_info.blue_primary.y=(double) ReadBlobLSBLong(image)/BMP_DENOM;
             bmp_info.blue_primary.z=(double) ReadBlobLSBLong(image)/BMP_DENOM;
 
-            sum=bmp_info.red_primary.x+bmp_info.red_primary.y+
+            gamma=bmp_info.red_primary.x+bmp_info.red_primary.y+
               bmp_info.red_primary.z;
-            bmp_info.red_primary.x/=sum;
-            bmp_info.red_primary.y/=sum;
+            gamma=PerceptibleReciprocal(gamma);
+            bmp_info.red_primary.x*=gamma;
+            bmp_info.red_primary.y*=gamma;
             image->chromaticity.red_primary.x=bmp_info.red_primary.x;
             image->chromaticity.red_primary.y=bmp_info.red_primary.y;
 
-            sum=bmp_info.green_primary.x+bmp_info.green_primary.y+
+            gamma=bmp_info.green_primary.x+bmp_info.green_primary.y+
               bmp_info.green_primary.z;
-            bmp_info.green_primary.x/=sum;
-            bmp_info.green_primary.y/=sum;
+            gamma=PerceptibleReciprocal(gamma);
+            bmp_info.green_primary.x*=gamma;
+            bmp_info.green_primary.y*=gamma;
             image->chromaticity.green_primary.x=bmp_info.green_primary.x;
             image->chromaticity.green_primary.y=bmp_info.green_primary.y;
 
-            sum=bmp_info.blue_primary.x+bmp_info.blue_primary.y+
+            gamma=bmp_info.blue_primary.x+bmp_info.blue_primary.y+
               bmp_info.blue_primary.z;
-            bmp_info.blue_primary.x/=sum;
-            bmp_info.blue_primary.y/=sum;
+            gamma=PerceptibleReciprocal(gamma);
+            bmp_info.blue_primary.x*=gamma;
+            bmp_info.blue_primary.y*=gamma;
             image->chromaticity.blue_primary.x=bmp_info.blue_primary.x;
             image->chromaticity.blue_primary.y=bmp_info.blue_primary.y;
 
@@ -1519,7 +1528,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
     bmp_info;
 
   const char
-    *value;
+    *option;
 
   const StringInfo
     *profile;
@@ -1577,18 +1586,17 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
     if (LocaleCompare(image_info->magick,"BMP3") == 0)
       type=3;
 
-  value=GetImageOption(image_info,"bmp:format");
-
-  if (value != (char *) NULL)
+  option=GetImageOption(image_info,"bmp:format");
+  if (option != (char *) NULL)
     {
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "  Format=%s",value);
+          "  Format=%s",option);
 
-      if (LocaleCompare(value,"bmp2") == 0)
+      if (LocaleCompare(option,"bmp2") == 0)
         type=2;
-      if (LocaleCompare(value,"bmp3") == 0)
+      if (LocaleCompare(option,"bmp3") == 0)
         type=3;
-      if (LocaleCompare(value,"bmp4") == 0)
+      if (LocaleCompare(option,"bmp4") == 0)
         type=4;
     }
 
@@ -1650,6 +1658,12 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
           ((type > 3) && (image->matte != MagickFalse) ? 32 : 24);
         bmp_info.compression=(unsigned int) ((type > 3) &&
           (image->matte != MagickFalse) ?  BI_BITFIELDS : BI_RGB);
+        if ((type == 3) && (image->matte != MagickFalse))
+          {
+            option=GetImageOption(image_info,"bmp3:alpha");
+            if (IsStringTrue(option))
+              bmp_info.bits_per_pixel=32;
+          }
       }
     bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
     bmp_info.ba_offset=0;
@@ -1969,7 +1983,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
            default:
            {
              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-               "   Compression=UNKNOWN (%u)",bmp_info.compression);
+               "   Compression=UNKNOWN (%lu)",bmp_info.compression);
              break;
            }
         }
@@ -1978,7 +1992,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
             "   Number_colors=unspecified");
         else
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-            "   Number_colors=%u",bmp_info.number_colors);
+            "   Number_colors=%lu",bmp_info.number_colors);
       }
     (void) WriteBlob(image,2,(unsigned char *) "BM");
     (void) WriteBlobLSBLong(image,bmp_info.file_size);
@@ -2132,7 +2146,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
       }
     if (image->debug != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-        "  Pixels:  %u bytes",bmp_info.image_size);
+        "  Pixels:  %lu bytes",bmp_info.image_size);
     (void) WriteBlob(image,(size_t) bmp_info.image_size,pixels);
     pixel_info=RelinquishVirtualMemory(pixel_info);
     if (GetNextImageInList(image) == (Image *) NULL)
@@ -2145,7 +2159,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image)
   } while (image_info->adjoin != MagickFalse);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-       "  File_size:  %u bytes",bmp_info.file_size);
+       "  File_size:  %lu bytes",bmp_info.file_size);
   (void) CloseBlob(image);
   return(MagickTrue);
 }
