@@ -17,7 +17,7 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -290,13 +290,6 @@ static CompositeOperator GIMPBlendModeToCompositeOperator(
 %
 */
 
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max)
 {
   int
@@ -471,119 +464,117 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
               length=(size_t) ((*xcfdata << 8) + xcfdata[1]);
               xcfdata+=2;
             }
-            size-=length;
-            if (size < 0)
-              goto bogus_rle;
-            if (&xcfdata[length-1] > xcfdatalimit)
-              goto bogus_rle;
-            while (length-- > 0)
-            {
-              data=(*xcfdata++);
-              switch (i)
-              {
-                case 0:
-                {
-                  SetPixelRed(q,ScaleCharToQuantum(data));
-                  if (inDocInfo->image_type != GIMP_GRAY)
-                    {
-                      SetPixelGreen(q,ScaleCharToQuantum(data));
-                      SetPixelBlue(q,ScaleCharToQuantum(data));
-                      SetPixelAlpha(q,data == 255U ? alpha :
-                        ScaleCharToQuantum(data));
-                    }
-                  else
-                    {
-                      SetPixelGreen(q,GetPixelRed(q));
-                      SetPixelBlue(q,GetPixelRed(q));
-                      SetPixelAlpha(q,data == 255U ? alpha :
-                        ScaleCharToQuantum(data));
-                    }
-                  break;
-                }
-                case 1:
-                {
-                  SetPixelGreen(q,ScaleCharToQuantum(data));
-                  break;
-                }
-                case 2:
-                {
-                  SetPixelBlue(q,ScaleCharToQuantum(data));
-                  break;
-                }
-                case 3:
-                {
-                  SetPixelAlpha(q,data == 255U ? alpha :
-                    ScaleCharToQuantum(data));
-                  break;
-                }
-              }
-              q++;
-            }
-          }
-        else
+          size-=length;
+          if (size < 0)
+            goto bogus_rle;
+          if (&xcfdata[length-1] > xcfdatalimit)
+            goto bogus_rle;
+          while (length-- > 0)
           {
-            length+=1;
-            if (length == 128)
-              {
-                if (xcfdata >= xcfdatalimit)
-                  goto bogus_rle;
-                length=(size_t) ((*xcfdata << 8) + xcfdata[1]);
-                xcfdata+=2;
-              }
-            size-=length;
-            if (size < 0)
-              goto bogus_rle;
-            if (xcfdata > xcfdatalimit)
-              goto bogus_rle;
-            pixel=(*xcfdata++);
-            for (j=0; j < (ssize_t) length; j++)
+            data=(*xcfdata++);
+            switch (i)
             {
-              data=pixel;
-              switch (i)
+              case 0:
               {
-                case 0:
-                {
-                  SetPixelRed(q,ScaleCharToQuantum(data));
-                  if (inDocInfo->image_type != GIMP_GRAY)
-                    {
-                      SetPixelGreen(q,ScaleCharToQuantum(data));
-                      SetPixelBlue(q,ScaleCharToQuantum(data));
-                      SetPixelAlpha(q,data == 255U ? alpha :
-                        ScaleCharToQuantum(data));
-                    }
-                  else
-                    {
-                      SetPixelGreen(q,GetPixelRed(q));
-                      SetPixelBlue(q,GetPixelRed(q));
-                      SetPixelAlpha(q,data == 255U ? alpha :
-                        ScaleCharToQuantum(data));
-                    }
-                  break;
-                }
-                case 1:
-                {
+                SetPixelRed(q,ScaleCharToQuantum(data));
+                if (inDocInfo->image_type == GIMP_GRAY)
+                  {
+                    SetPixelGreen(q,ScaleCharToQuantum(data));
+                    SetPixelBlue(q,ScaleCharToQuantum(data));
+                  }
+                else
+                  {
+                    SetPixelGreen(q,GetPixelRed(q));
+                    SetPixelBlue(q,GetPixelRed(q));
+                  }
+                SetPixelAlpha(q,alpha);
+                break;
+              }
+              case 1:
+              {
+                if (inDocInfo->image_type == GIMP_GRAY)
+                  SetPixelAlpha(q,ScaleCharToQuantum(data));
+                else
                   SetPixelGreen(q,ScaleCharToQuantum(data));
-                  break;
-                }
-                case 2:
-                {
-                  SetPixelBlue(q,ScaleCharToQuantum(data));
-                  break;
-                }
-                case 3:
-                {
-                  SetPixelAlpha(q,data == 255U ? alpha :
-                    ScaleCharToQuantum(data));
-                  break;
-                }
+                break;
+              }
+              case 2:
+              {
+                SetPixelBlue(q,ScaleCharToQuantum(data));
+                break;
+              }
+              case 3:
+              {
+                SetPixelAlpha(q,ScaleCharToQuantum(data));
+                break;
               }
-              q++;
             }
+            q++;
           }
-      }
-      if (SyncAuthenticPixels(tile_image,exception) == MagickFalse)
-        break;
+        }
+      else
+        {
+          length+=1;
+          if (length == 128)
+            {
+              if (xcfdata >= xcfdatalimit)
+                goto bogus_rle;
+              length=(size_t) ((*xcfdata << 8) + xcfdata[1]);
+              xcfdata+=2;
+            }
+          size-=length;
+          if (size < 0)
+            goto bogus_rle;
+          if (xcfdata > xcfdatalimit)
+            goto bogus_rle;
+          pixel=(*xcfdata++);
+          for (j=0; j < (ssize_t) length; j++)
+          {
+            data=pixel;
+            switch (i)
+            {
+              case 0:
+              {
+                SetPixelRed(q,ScaleCharToQuantum(data));
+                if (inDocInfo->image_type == GIMP_GRAY)
+                  {
+                    SetPixelGreen(q,ScaleCharToQuantum(data));
+                    SetPixelBlue(q,ScaleCharToQuantum(data));
+                  }
+                else
+                  {
+                    SetPixelGreen(q,GetPixelRed(q));
+                    SetPixelBlue(q,GetPixelRed(q));
+                  }
+                SetPixelAlpha(q,alpha);
+                break;
+              }
+              case 1:
+              {
+                if (inDocInfo->image_type == GIMP_GRAY)
+                  SetPixelAlpha(q,ScaleCharToQuantum(data));
+                else
+                  SetPixelGreen(q,ScaleCharToQuantum(data));
+                break;
+              }
+              case 2:
+              {
+                SetPixelBlue(q,ScaleCharToQuantum(data));
+                break;
+              }
+              case 3:
+              {
+                SetPixelAlpha(q,ScaleCharToQuantum(data));
+                break;
+              }
+            }
+            q++;
+          }
+        }
     }
+    if (SyncAuthenticPixels(tile_image,exception) == MagickFalse)
+      break;
+  }
   xcfodata=(unsigned char *) RelinquishMagickMemory(xcfodata);
   return(MagickTrue);
 
@@ -1051,7 +1042,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   count=ReadBlob(image,14,(unsigned char *) magick);
-  if ((count == 0) ||
+  if ((count != 14) ||
       (LocaleNCompare((char *) magick,"gimp xcf",8) != 0))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   (void) ResetMagickMemory(&doc_info,0,sizeof(XCFDocInfo));
@@ -1277,6 +1268,12 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       XCFLayerInfo
         *layer_info;
 
+      status=SetImageExtent(image,image->columns,image->rows);
+      if (status == MagickFalse)
+        {
+          InheritException(exception,&image->exception);
+          return(DestroyImageList(image));
+        }
       /*
         The read pointer.
       */
