@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
@@ -18,33 +18,28 @@
 #ifndef _MAGICKCORE_ACCELERATE_H
 #define _MAGICKCORE_ACCELERATE_H
 
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
+#include "magick/fx.h"
 #include "magick/morphology.h"
 #include "magick/resample.h"
 #include "magick/resize.h"
 #include "magick/statistic.h"
 
-extern MagickExport MagickBooleanType
-  AccelerateContrastImage(Image *,const MagickBooleanType,ExceptionInfo *),
-  AccelerateConvolveImage(const Image *,const KernelInfo *,Image *,
-    ExceptionInfo *),
-  AccelerateEqualizeImage(Image *,const ChannelType,ExceptionInfo *),
-  AccelerateFunctionImage(Image *,const ChannelType,const MagickFunction,
-    const size_t,const double *,ExceptionInfo *),
-  AccelerateModulateImage(Image* image, double percent_brightness, 
-    double percent_hue, double percent_saturation, 
-    ColorspaceType colorspace, ExceptionInfo* exception);
-
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
 
 extern MagickExport Image
+  *AccelerateAddNoiseImage(const Image*,const ChannelType,const NoiseType,
+    ExceptionInfo *),
   *AccelerateBlurImage(const Image *,const ChannelType,const double,
     const double,ExceptionInfo *),
   *AccelerateConvolveImageChannel(const Image *,const ChannelType,
     const KernelInfo *,ExceptionInfo *),
   *AccelerateDespeckleImage(const Image *,ExceptionInfo *),
+  *AccelerateLocalContrastImage(const Image *,const double,const double,
+    ExceptionInfo *),
+  *AccelerateMotionBlurImage(const Image*, const ChannelType,
+    const double*,const size_t,const OffsetInfo*,ExceptionInfo*),
   *AccelerateRadialBlurImage(const Image *,const ChannelType,const double,
     ExceptionInfo *),
   *AccelerateResizeImage(const Image *,const size_t,const size_t,
@@ -52,6 +47,28 @@ extern MagickExport Image
   *AccelerateUnsharpMaskImage(const Image *,const ChannelType,const double,
     const double,const double,const double,ExceptionInfo *);
 
+extern MagickExport MagickBooleanType
+  AccelerateCompositeImage(Image *,const ChannelType,const CompositeOperator,
+    const Image *,const ssize_t,const ssize_t,const float,const float,ExceptionInfo *),
+  AccelerateContrastImage(Image *,const MagickBooleanType,ExceptionInfo *),
+  AccelerateContrastStretchImageChannel(Image *, const ChannelType, const double, const double, 
+    ExceptionInfo*),
+  AccelerateEqualizeImage(Image *,const ChannelType,ExceptionInfo *),
+  AccelerateFunctionImage(Image *,const ChannelType,const MagickFunction,
+    const size_t,const double *,ExceptionInfo *),
+  AccelerateGrayscaleImage(Image*, const PixelIntensityMethod,
+    ExceptionInfo *),
+  AccelerateModulateImage(Image*, double, double, double, 
+    ColorspaceType, ExceptionInfo*),
+  AccelerateRandomImage(Image*, ExceptionInfo*);
+
+/* legacy, do not use */
+extern MagickExport MagickBooleanType
+  AccelerateConvolveImage(const Image *,const KernelInfo *,Image *,
+    ExceptionInfo *),
+  AccelerateNegateImageChannel(Image*, const ChannelType, const MagickBooleanType,
+    ExceptionInfo *);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
