@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -68,6 +68,9 @@ extern MagickExport char
 extern MagickExport const struct stat
   *GetBlobProperties(const Image *);
 
+extern MagickExport const void
+  *ReadBlobStream(Image *,const size_t,void *,ssize_t *);
+
 extern MagickExport double
   ReadBlobDouble(Image *);
 
@@ -127,6 +130,9 @@ extern MagickExport void
   MSBOrderLong(unsigned char *,const size_t),
   MSBOrderShort(unsigned char *,const size_t);
 
+extern MagickPrivate void
+  DisassociateBlob(Image *);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
