@@ -17,7 +17,7 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -62,17 +62,12 @@
 /*
   Define declarations.
 */
+#if defined(MAGICKCORE_HAVE_NEWLOCALE) || defined(MAGICKCORE_WINDOWS_SUPPORT)
+#  define MAGICKCORE_LOCALE_SUPPORT
+#endif
 #define LocaleFilename  "locale.xml"
 #define MaxRecursionDepth  200
 
-/*
-  Typedef declarations.
-*/
-#if defined(__CYGWIN__)
-typedef struct _locale_t
-  *locale_t;
-#endif
-
 /*
   Static declarations.
 */
@@ -94,7 +89,7 @@ static SemaphoreInfo
 static SplayTreeInfo
   *locale_cache = (SplayTreeInfo *) NULL;
 
-#if defined(MAGICKCORE_HAVE_STRTOD_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT)
 static volatile locale_t
   c_locale = (locale_t) NULL;
 #endif
@@ -107,7 +102,7 @@ static MagickBooleanType
   LoadLocaleCache(SplayTreeInfo *,const char *,const char *,const char *,
     const size_t,ExceptionInfo *);
 
-#if defined(MAGICKCORE_HAVE_STRTOD_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -237,7 +232,7 @@ static SplayTreeInfo *AcquireLocaleSplayTree(const char *filename,
   return(locale_cache);
 }
 
-#if defined(MAGICKCORE_HAVE_STRTOD_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -334,12 +329,12 @@ MagickExport LinkedListInfo *DestroyLocaleOptions(LinkedListInfo *messages)
 */
 
 MagickExport ssize_t FormatLocaleFileList(FILE *file,
-  const char *restrict format,va_list operands)
+  const char *magick_restrict format,va_list operands)
 {
   ssize_t
     n;
 
-#if defined(MAGICKCORE_HAVE_VFPRINTF_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT) && defined(MAGICKCORE_HAVE_VFPRINTF_L)
   {
     locale_t
       locale;
@@ -355,7 +350,7 @@ MagickExport ssize_t FormatLocaleFileList(FILE *file,
 #endif
   }
 #else
-#if defined(MAGICKCORE_HAVE_USELOCALE) && defined(MAGICKCORE_HAVE_STRTOD_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT) && defined(MAGICKCORE_HAVE_USELOCALE)
   {
     locale_t
       locale,
@@ -378,8 +373,8 @@ MagickExport ssize_t FormatLocaleFileList(FILE *file,
   return(n);
 }
 
-MagickExport ssize_t FormatLocaleFile(FILE *file,const char *restrict format,
-  ...)
+MagickExport ssize_t FormatLocaleFile(FILE *file,
+  const char *magick_restrict format,...)
 {
   ssize_t
     n;
@@ -424,13 +419,13 @@ MagickExport ssize_t FormatLocaleFile(FILE *file,const char *restrict format,
 %
 */
 
-MagickExport ssize_t FormatLocaleStringList(char *restrict string,
-  const size_t length,const char *restrict format,va_list operands)
+MagickExport ssize_t FormatLocaleStringList(char *magick_restrict string,
+  const size_t length,const char *magick_restrict format,va_list operands)
 {
   ssize_t
     n;
 
-#if defined(MAGICKCORE_HAVE_VSNPRINTF_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT) && defined(MAGICKCORE_HAVE_VSNPRINTF_L)
   {
     locale_t
       locale;
@@ -446,7 +441,7 @@ MagickExport ssize_t FormatLocaleStringList(char *restrict string,
 #endif
   }
 #elif defined(MAGICKCORE_HAVE_VSNPRINTF)
-#if defined(MAGICKCORE_HAVE_USELOCALE) && defined(MAGICKCORE_HAVE_STRTOD_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT) && defined(MAGICKCORE_HAVE_USELOCALE)
   {
     locale_t
       locale,
@@ -473,8 +468,8 @@ MagickExport ssize_t FormatLocaleStringList(char *restrict string,
   return(n);
 }
 
-MagickExport ssize_t FormatLocaleString(char *restrict string,
-  const size_t length,const char *restrict format,...)
+MagickExport ssize_t FormatLocaleString(char *magick_restrict string,
+  const size_t length,const char *magick_restrict format,...)
 {
   ssize_t
     n;
@@ -976,8 +971,8 @@ static MagickBooleanType IsLocaleTreeInstantiated(ExceptionInfo *exception)
 %      referenced by sentinal.
 %
 */
-MagickExport double InterpretLocaleValue(const char *restrict string,
-  char **restrict sentinal)
+MagickExport double InterpretLocaleValue(const char *magick_restrict string,
+  char **magick_restrict sentinal)
 {
   char
     *q;
@@ -989,7 +984,7 @@ MagickExport double InterpretLocaleValue(const char *restrict string,
     value=(double) strtoul(string,&q,16);
   else
     {
-#if defined(MAGICKCORE_HAVE_STRTOD_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT) && defined(MAGICKCORE_HAVE_STRTOD_L)
       locale_t
         locale;
 
@@ -1383,6 +1378,204 @@ static MagickBooleanType LoadLocaleCache(SplayTreeInfo *locale_cache,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   L o c a l e C o m p a r e                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  LocaleCompare() performs a case-insensitive comparison of two strings
+%  byte-by-byte, according to the ordering of the current locale encoding.
+%  LocaleCompare returns an integer greater than, equal to, or less than 0,
+%  if the string pointed to by p is greater than, equal to, or less than the
+%  string pointed to by q respectively.  The sign of a non-zero return value
+%  is determined by the sign of the difference between the values of the first
+%  pair of bytes that differ in the strings being compared.
+%
+%  The format of the LocaleCompare method is:
+%
+%      int LocaleCompare(const char *p,const char *q)
+%
+%  A description of each parameter follows:
+%
+%    o p: A pointer to a character string.
+%
+%    o q: A pointer to a character string to compare to p.
+%
+*/
+MagickExport int LocaleCompare(const char *p,const char *q)
+{
+  if ((p == (char *) NULL) && (q == (char *) NULL))
+    return(0);
+  if (p == (char *) NULL)
+    return(-1);
+  if (q == (char *) NULL)
+    return(1);
+#if defined(MAGICKCORE_HAVE_STRCASECMP)
+  return(strcasecmp(p,q));
+#else
+  {
+    register int
+      c,
+      d;
+
+    for ( ; ; )
+    {
+      c=(int) *((unsigned char *) p);
+      d=(int) *((unsigned char *) q);
+      if ((c == 0) || (AsciiMap[c] != AsciiMap[d]))
+        break;
+      p++;
+      q++;
+    }
+    return(AsciiMap[c]-(int) AsciiMap[d]);
+  }
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   L o c a l e L o w e r                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  LocaleLower() transforms all of the characters in the supplied
+%  null-terminated string, changing all uppercase letters to lowercase.
+%
+%  The format of the LocaleLower method is:
+%
+%      void LocaleLower(char *string)
+%
+%  A description of each parameter follows:
+%
+%    o string: A pointer to the string to convert to lower-case Locale.
+%
+*/
+MagickExport void LocaleLower(char *string)
+{
+  register char
+    *q;
+
+  assert(string != (char *) NULL);
+  for (q=string; *q != '\0'; q++)
+    *q=(char) tolower((int) *q);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   L o c a l e N C o m p a r e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  LocaleNCompare() performs a case-insensitive comparison of two strings
+%  byte-by-byte, according to the ordering of the current locale encoding.
+%
+%  LocaleNCompare returns an integer greater than, equal to, or less than 0,
+%  if the string pointed to by p is greater than, equal to, or less than the
+%  string pointed to by q respectively.  The sign of a non-zero return value
+%  is determined by the sign of the difference between the values of the first
+%  pair of bytes that differ in the strings being compared.
+%
+%  The LocaleNCompare method makes the same comparison as LocaleCompare but
+%  looks at a maximum of n bytes.  Bytes following a null byte are not
+%  compared.
+%
+%  The format of the LocaleNCompare method is:
+%
+%      int LocaleNCompare(const char *p,const char *q,const size_t n)
+%
+%  A description of each parameter follows:
+%
+%    o p: A pointer to a character string.
+%
+%    o q: A pointer to a character string to compare to p.
+%
+%    o length: the number of characters to compare in strings p and q.
+%
+*/
+MagickExport int LocaleNCompare(const char *p,const char *q,const size_t length)
+{
+  if ((p == (char *) NULL) && (q == (char *) NULL))
+    return(0);
+  if (p == (char *) NULL)
+    return(-1);
+  if (q == (char *) NULL)
+    return(1);
+#if defined(MAGICKCORE_HAVE_STRNCASECMP)
+  return(strncasecmp(p,q,length));
+#else
+  {
+    register int
+      c,
+      d;
+
+    register size_t
+      i;
+
+    for (i=length; i != 0; i--)
+    {
+      c=(int) *((unsigned char *) p);
+      d=(int) *((unsigned char *) q);
+      if (AsciiMap[c] != AsciiMap[d])
+        return(AsciiMap[c]-(int) AsciiMap[d]);
+      if (c == 0)
+        return(0);
+      p++;
+      q++;
+    }
+    return(0);
+  }
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   L o c a l e U p p e r                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  LocaleUpper() transforms all of the characters in the supplied
+%  null-terminated string, changing all lowercase letters to uppercase.
+%
+%  The format of the LocaleUpper method is:
+%
+%      void LocaleUpper(char *string)
+%
+%  A description of each parameter follows:
+%
+%    o string: A pointer to the string to convert to upper-case Locale.
+%
+*/
+MagickExport void LocaleUpper(char *string)
+{
+  register char
+    *q;
+
+  assert(string != (char *) NULL);
+  for (q=string; *q != '\0'; q++)
+    *q=(char) toupper((int) *q);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 +   L o c a l e C o m p o n e n t G e n e s i s                               %
 %                                                                             %
 %                                                                             %
@@ -1428,7 +1621,7 @@ MagickExport void LocaleComponentTerminus(void)
   LockSemaphoreInfo(locale_semaphore);
   if (locale_cache != (SplayTreeInfo *) NULL)
     locale_cache=DestroySplayTree(locale_cache);
-#if defined(MAGICKCORE_HAVE_STRTOD_L)
+#if defined(MAGICKCORE_LOCALE_SUPPORT)
   DestroyCLocale();
 #endif
   UnlockSemaphoreInfo(locale_semaphore);
