@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -18,13 +18,13 @@
 #ifndef _MAGICKCORE_LOG_H
 #define _MAGICKCORE_LOG_H
 
+#include <stdarg.h>
+#include "magick/exception.h"
+
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-#include <stdarg.h>
-#include "magick/exception.h"
-
 #if !defined(GetMagickModule)
 # define GetMagickModule()  __FILE__,__func__,(unsigned long) __LINE__
 #endif
@@ -60,13 +60,16 @@ typedef enum
 typedef struct _LogInfo
   LogInfo;
 
+typedef void
+  (*MagickLogMethod)(const LogEventType,const char *);
+
 extern MagickExport char
   **GetLogList(const char *,size_t *,ExceptionInfo *);
 
 extern MagickExport const char
   *GetLogName(void),
   *SetLogName(const char *);
-                                                                                
+
 extern MagickExport const LogInfo
   **GetLogInfoList(const char *,size_t *,ExceptionInfo *);
 
@@ -86,7 +89,8 @@ extern MagickExport MagickBooleanType
 extern MagickExport void
   CloseMagickLog(void),
   LogComponentTerminus(void),
-  SetLogFormat(const char *);
+  SetLogFormat(const char *),
+  SetLogMethod(MagickLogMethod);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
