@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -848,7 +848,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               target_profile;
 
             cmsHTRANSFORM
-              *restrict transform;
+              *magick_restrict transform;
 
             cmsUInt32Number
               flags,
@@ -875,8 +875,8 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               y;
 
             unsigned short
-              **restrict source_pixels,
-              **restrict target_pixels;
+              **magick_restrict source_pixels,
+              **magick_restrict target_pixels;
 
             exception=(&image->exception);
             target_profile=(cmsHPROFILE) NULL;
@@ -1098,13 +1098,13 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 sync;
 
               register IndexPacket
-                *restrict indexes;
+                *magick_restrict indexes;
 
               register ssize_t
                 x;
 
               register PixelPacket
-                *restrict q;
+                *magick_restrict q;
 
               register unsigned short
                 *p;
@@ -1534,7 +1534,8 @@ static void GetProfilesFromResourceBlock(Image *image,
       break;
     p=ReadResourceLong(p,&value);
     count=(ssize_t) value;
-    if ((p > (datum+length-count)) || (count > (ssize_t) length))
+    if ((p > (datum+length-count)) || (count > (ssize_t) length) ||
+        (count < 0))
       break;
     switch (id)
     {
@@ -1874,13 +1875,13 @@ static MagickBooleanType Sync8BimProfile(Image *image,StringInfo *profile)
       return(MagickFalse);
     id=ReadProfileMSBShort(&p,&length);
     count=(ssize_t) ReadProfileByte(&p,&length);
-    if (count > (ssize_t) length)
+    if ((count > (ssize_t) length) || (count < 0))
       return(MagickFalse);
     p+=count;
     if ((*p & 0x01) == 0)
       (void) ReadProfileByte(&p,&length);
     count=(ssize_t) ReadProfileMSBLong(&p,&length);
-    if (count > (ssize_t) length)
+    if ((count > (ssize_t) length) || (count < 0))
       return(MagickFalse);
     if ((id == 0x3ED) && (count == 16))
       {
