@@ -17,14 +17,13 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the license.  You may  %
 %  obtain a copy of the license at                                            %
 %                                                                             %
 %    http://www.imagemagick.org/script/license.php                            %
-%                                                                             %
 %  unless required by applicable law or agreed to in writing, software        %
 %  distributed under the license is distributed on an "as is" basis,          %
 %  without warranties or conditions of any kind, either express or implied.   %
@@ -44,6 +43,7 @@
 #include "magick/blob-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/locale_.h"
 #include "magick/log.h"
@@ -214,10 +214,15 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
     *string_info;
 
   string_info=AcquireStringInfo(0);
+  if (~length < MaxTextExtent)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   string_info->length=length;
-  if (~string_info->length >= (MaxTextExtent-1))
-    string_info->datum=(unsigned char *) AcquireQuantumMemory(
-      string_info->length+MaxTextExtent,sizeof(*string_info->datum));
+  if (string_info->datum == (unsigned char *) NULL)
+    string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
+      MaxTextExtent,sizeof(*string_info->datum));
+  else
+    string_info->datum=(unsigned char *) ResizeQuantumMemory(string_info->datum,
+      length+MaxTextExtent,sizeof(*string_info->datum));
   if (string_info->datum == (unsigned char *) NULL)
     {
       string_info=DestroyStringInfo(string_info);
@@ -352,13 +357,6 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
 %
 */
 
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 MagickExport int CompareStringInfo(const StringInfo *target,
   const StringInfo *source)
 {
@@ -650,6 +648,9 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
   string_info=AcquireStringInfo(0);
   (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
   string_info->length=length;
+  if (string_info->datum != (unsigned char *) NULL)
+    string_info->datum=(unsigned char *) RelinquishMagickMemory( 
+      string_info->datum);
   string_info->datum=(unsigned char *) string;
   return(string_info);
 }
@@ -937,17 +938,14 @@ MagickExport char *EscapeString(const char *source,const char escape)
   if (destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToEscapeString");
   *destination='\0';
-  if (source != (char *) NULL)
-    {
-      q=destination;
-      for (p=source; *p != '\0'; p++)
-      {
-        if ((*p == '\\') || (*p == escape))
-          *q++='\\';
-        *q++=(*p);
-      }
-      *q='\0';
-    }
+  q=destination;
+  for (p=source; *p != '\0'; p++)
+  {
+    if ((*p == '\\') || (*p == escape))
+      *q++='\\';
+    *q++=(*p);
+  }
+  *q='\0';
   return(destination);
 }
 
@@ -1027,6 +1025,9 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
   assert(exception != (ExceptionInfo *) NULL);
   string_info=AcquireStringInfo(0);
   (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
+  if (string_info->datum != (unsigned char *) NULL)
+    string_info->datum=(unsigned char *) RelinquishMagickMemory( 
+      string_info->datum);
   string_info->datum=FileToBlob(filename,extent,&string_info->length,exception);
   if (string_info->datum == (unsigned char *) NULL)
     {
@@ -1344,8 +1345,8 @@ MagickExport const char *GetStringInfoPath(const StringInfo *string_info)
 %      after the last character used in the conversion.
 %
 */
-MagickExport double InterpretSiPrefixValue(const char *restrict string,
-  char **restrict sentinal)
+MagickExport double InterpretSiPrefixValue(const char *magick_restrict string,
+  char **magick_restrict sentinal)
 {
   char
     *q;
@@ -1399,7 +1400,7 @@ MagickExport double InterpretSiPrefixValue(const char *restrict string,
                 }
             }
         }
-      if (*q == 'B')
+      if ((*q == 'B') || (*q == 'P'))
         q++;
     }
   if (sentinal != (char **) NULL)
@@ -1495,204 +1496,6 @@ MagickExport MagickBooleanType IsStringNotFalse(const char *value)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   L o c a l e C o m p a r e                                                 %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  LocaleCompare() performs a case-insensitive comparison of two strings
-%  byte-by-byte, according to the ordering of the current locale encoding.
-%  LocaleCompare returns an integer greater than, equal to, or less than 0,
-%  if the string pointed to by p is greater than, equal to, or less than the
-%  string pointed to by q respectively.  The sign of a non-zero return value
-%  is determined by the sign of the difference between the values of the first
-%  pair of bytes that differ in the strings being compared.
-%
-%  The format of the LocaleCompare method is:
-%
-%      int LocaleCompare(const char *p,const char *q)
-%
-%  A description of each parameter follows:
-%
-%    o p: A pointer to a character string.
-%
-%    o q: A pointer to a character string to compare to p.
-%
-*/
-MagickExport int LocaleCompare(const char *p,const char *q)
-{
-  if ((p == (char *) NULL) && (q == (char *) NULL))
-    return(0);
-  if (p == (char *) NULL)
-    return(-1);
-  if (q == (char *) NULL)
-    return(1);
-#if defined(MAGICKCORE_HAVE_STRCASECMP)
-  return(strcasecmp(p,q));
-#else
-  {
-    register int
-      c,
-      d;
-
-    for ( ; ; )
-    {
-      c=(int) *((unsigned char *) p);
-      d=(int) *((unsigned char *) q);
-      if ((c == 0) || (AsciiMap[c] != AsciiMap[d]))
-        break;
-      p++;
-      q++;
-    }
-    return(AsciiMap[c]-(int) AsciiMap[d]);
-  }
-#endif
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   L o c a l e L o w e r                                                     %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  LocaleLower() transforms all of the characters in the supplied
-%  null-terminated string, changing all uppercase letters to lowercase.
-%
-%  The format of the LocaleLower method is:
-%
-%      void LocaleLower(char *string)
-%
-%  A description of each parameter follows:
-%
-%    o string: A pointer to the string to convert to lower-case Locale.
-%
-*/
-MagickExport void LocaleLower(char *string)
-{
-  register char
-    *q;
-
-  assert(string != (char *) NULL);
-  for (q=string; *q != '\0'; q++)
-    *q=(char) tolower((int) *q);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   L o c a l e N C o m p a r e                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  LocaleNCompare() performs a case-insensitive comparison of two strings
-%  byte-by-byte, according to the ordering of the current locale encoding.
-%
-%  LocaleNCompare returns an integer greater than, equal to, or less than 0,
-%  if the string pointed to by p is greater than, equal to, or less than the
-%  string pointed to by q respectively.  The sign of a non-zero return value
-%  is determined by the sign of the difference between the values of the first
-%  pair of bytes that differ in the strings being compared.
-%
-%  The LocaleNCompare method makes the same comparison as LocaleCompare but
-%  looks at a maximum of n bytes.  Bytes following a null byte are not
-%  compared.
-%
-%  The format of the LocaleNCompare method is:
-%
-%      int LocaleNCompare(const char *p,const char *q,const size_t n)
-%
-%  A description of each parameter follows:
-%
-%    o p: A pointer to a character string.
-%
-%    o q: A pointer to a character string to compare to p.
-%
-%    o length: the number of characters to compare in strings p and q.
-%
-*/
-MagickExport int LocaleNCompare(const char *p,const char *q,const size_t length)
-{
-  if ((p == (char *) NULL) && (q == (char *) NULL))
-    return(0);
-  if (p == (char *) NULL)
-    return(-1);
-  if (q == (char *) NULL)
-    return(1);
-#if defined(MAGICKCORE_HAVE_STRNCASECMP)
-  return(strncasecmp(p,q,length));
-#else
-  {
-    register int
-      c,
-      d;
-
-    register size_t
-      i;
-
-    for (i=length; i != 0; i--)
-    {
-      c=(int) *((unsigned char *) p);
-      d=(int) *((unsigned char *) q);
-      if (AsciiMap[c] != AsciiMap[d])
-        return(AsciiMap[c]-(int) AsciiMap[d]);
-      if (c == 0)
-        return(0);
-      p++;
-      q++;
-    }
-    return(0);
-  }
-#endif
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   L o c a l e U p p e r                                                     %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  LocaleUpper() transforms all of the characters in the supplied
-%  null-terminated string, changing all lowercase letters to uppercase.
-%
-%  The format of the LocaleUpper method is:
-%
-%      void LocaleUpper(char *string)
-%
-%  A description of each parameter follows:
-%
-%    o string: A pointer to the string to convert to upper-case Locale.
-%
-*/
-MagickExport void LocaleUpper(char *string)
-{
-  register char
-    *q;
-
-  assert(string != (char *) NULL);
-  for (q=string; *q != '\0'; q++)
-    *q=(char) toupper((int) *q);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   P r i n t S t r i n g I n f o                                             %
 %                                                                             %
 %                                                                             %
